import 'package:test/test.dart';

import 'shared/integration_test_set.dart';

/// The sample 3 was generated by `coverage` (for dart instead of Flutter). It created a lcov file using absolute paths
void main() {
  const lcovFileParam = ["--lcov-file", "input_samples/sample3/lcov.info"];
  const diffFilePath = "input_samples/sample3/git.diff";
  group("when running with default settings", () {
    test("report 102 lines that should be tested", () async {
      final analyze = await getAnalyzeForIntegrationTest(
        arguments: lcovFileParam,
        diffFilePath: diffFilePath,
      );
      final result = await analyze();
      expect(result.linesShouldBeTested, 102);
    });

    test("report 26 lines missing test", () async {
      final analyze = await getAnalyzeForIntegrationTest(
        arguments: lcovFileParam,
        diffFilePath: diffFilePath,
      );
      final result = await analyze();
      expect(result.linesMissingTests, 26);
    });
  });

  group("When `addLine` annotation is ignored", () {
    test("report 101 lines that should be tested", () async {
      final analyze = await getAnalyzeForIntegrationTest(
        arguments: [...lcovFileParam, "--ignore-lines", "^.*addLine.*\$"],
        diffFilePath: diffFilePath,
      );
      final result = await analyze();
      expect(result.linesShouldBeTested, 101);
    });

    test("report 1 untested ignored lines", () async {
      final analyze = await getAnalyzeForIntegrationTest(
        arguments: [...lcovFileParam, "--ignore-lines", "^.*addLine.*\$"],
        diffFilePath: diffFilePath,
      );
      final result = await analyze();
      expect(result.untestedIgnoredLines, 1);
    });
  });
}
